#include "ppc.h"

#include "m33.h"

#include <math.h>

PPC::PPC(float hfov, int _w, int _h) : w(_w), h(_h) {

  C = V3(0.0f, 0.0f, 0.0f);
  a = V3(1.0f, 0.0f, 0.0f);
  b = V3(0.0f, -1.0f, 0.0f);
  float hfovRAD = hfov / 180.0f * 3.1415926f;
  c = V3(-((float)w)/2.0f, ((float)h)/2.0f, -(float)w/(2*tanf(hfovRAD/2.0f)));

}

bool PPC::Project(V3 P, V3 &projP) {

  M33 cam;
  cam.SetColumn(0, a);
  cam.SetColumn(1, b);
  cam.SetColumn(2, c);
  V3 Q = cam.Invert()*(P-C);

  if (Q[2] <= 0.0f)
    return false;

  projP[0] = Q[0] / Q[2];
  projP[1] = Q[1] / Q[2];
  projP[2] = 1.0f / Q[2];

  return true;

}

void PPC::TranslateRight(float tstep) {

  C = C + a.Normalized()*tstep;

}

void PPC::TranslateLeft(float tstep) {

  C = C - a.Normalized()*tstep;

}

void PPC::TranslateUp(float tstep) {

  C = C - b.Normalized()*tstep;

}

void PPC::TranslateDown(float tstep) {

  C = C + b.Normalized()*tstep;

}

void PPC::Pan(float rstep) {

  a = a.RotateDirectionAboutArbitraryAxis(b.Normalized()*-1.0f, rstep);
  b = b.RotateDirectionAboutArbitraryAxis(b.Normalized()*-1.0f, rstep);
  c = c.RotateDirectionAboutArbitraryAxis(b.Normalized()*-1.0f, rstep);

}


V3 PPC::GetVD() {

  return (a^b).Normalized();

}